@@ -138,3 +138,35 @@ def is_instrinsic(input):
         return key == "Ref" or key == "Condition" or key.startswith("Fn::")
 
     return False
+
+
+def is_intrinsic_if(input):
+    """
+    Is the given input an intrinsic if? Intrinsic function 'if' is a dictionary with single
+    key - if
+
+    :param input: Input value to check if it is an intrinsic if
+    :return: True, if yes
+    """
+
+    if not is_instrinsic(input):
+        return False
+
+    key = list(input.keys())[0]
+    return key == "Fn::If"
+
+
+def is_intrinsic_no_value(input):
+    """
+    Is the given input an intrinsic Ref: AWS::NoValue? Intrinsic function is a dictionary with single
+    key - Ref and value - AWS::NoValue
+
+    :param input: Input value to check if it is an intrinsic if
+    :return: True, if yes
+    """
+
+    if not is_instrinsic(input):
+        return False
+
+    key = list(input.keys())[0]
+    return key == "Ref" and input["Ref"] == "AWS::NoValue"
